@@ -32,7 +32,6 @@ import org.apache.maven.artifact.Artifact;
 import org.apache.maven.artifact.factory.ArtifactFactory;
 import org.apache.maven.artifact.metadata.ArtifactMetadataRetrievalException;
 import org.apache.maven.artifact.metadata.ArtifactMetadataSource;
-import org.apache.maven.artifact.repository.ArtifactRepository;
 import org.apache.maven.artifact.resolver.ArtifactNotFoundException;
 import org.apache.maven.artifact.resolver.ArtifactResolutionException;
 import org.apache.maven.artifact.resolver.ArtifactResolver;
@@ -40,9 +39,12 @@ import org.apache.maven.artifact.versioning.ArtifactVersion;
 import org.apache.maven.artifact.versioning.InvalidVersionSpecificationException;
 import org.apache.maven.artifact.versioning.OverConstrainedVersionException;
 import org.apache.maven.artifact.versioning.VersionRange;
+import org.apache.maven.execution.MavenSession;
 import org.apache.maven.plugin.AbstractMojo;
 import org.apache.maven.plugin.MojoExecutionException;
 import org.apache.maven.plugin.MojoFailureException;
+import org.apache.maven.plugins.annotations.Component;
+import org.apache.maven.plugins.annotations.Parameter;
 import org.apache.maven.project.MavenProject;
 import org.codehaus.plexus.util.StringUtils;
 
@@ -66,101 +68,94 @@ abstract class AbstractBaselinePlugin
 
     /**
      * Flag to easily skip execution.
-     *
-     * @parameter expression="${baseline.skip}" default-value="false"
      */
+    @Parameter( property = "baseline.skip", defaultValue = "false" )
     protected boolean skip;
 
     /**
      * Whether to fail on errors.
-     *
-     * @parameter expression="${baseline.failOnError}" default-value="true"
      */
+    @Parameter( property = "baseline.failOnError", defaultValue = "true" )
     protected boolean failOnError;
 
     /**
      * Whether to fail on warnings.
-     *
-     * @parameter expression="${baseline.failOnWarning}" default-value="false"
      */
+    @Parameter( property = "baseline.failOnWarning", defaultValue = "false" )
     protected boolean failOnWarning;
 
-    /**
-     * @parameter expression="${project}"
-     * @required
-     * @readonly
-     */
+    @Parameter( defaultValue = "${project}", readonly = true, required = true )
     protected MavenProject project;
 
-    /**
-     * @parameter expression="${project.build.directory}"
-     * @required
-     * @readonly
-     */
+    @Parameter( defaultValue = "${session}", readonly = true, required = true )
+    protected MavenSession session;
+
+    @Parameter( defaultValue = "${project.build.directory}", readonly = true, required = true )
     private File buildDirectory;
 
-    /**
-     * @parameter expression="${project.build.finalName}"
-     * @required
-     * @readonly
-     */
+    @Parameter( defaultValue = "${project.build.finalName}", readonly = true, required = true )
     private String finalName;
 
-    /**
-     * @component
-     */
+    @Component
     protected ArtifactResolver resolver;
 
-    /**
-     * @component
-     */
+    @Component
     protected ArtifactFactory factory;
 
-    /**
-     * @parameter default-value="${localRepository}"
-     * @required
-     * @readonly
-     */
-    protected ArtifactRepository localRepository;
-
-    /**
-     * @component
-     */
+    @Component
     private ArtifactMetadataSource metadataSource;
 
     /**
-     * Version to compare the current code against.
-     *
-     * @parameter expression="${comparisonVersion}" default-value="(,${project.version})"
-     * @required
-     * @readonly
+     * Group id to compare the current code against.
      */
+    @Parameter( defaultValue = "${project.groupId}", property="comparisonGroupId" )
+    protected String comparisonGroupId;
+
+    /**
+     * Artifact to compare the current code against.
+     */
+    @Parameter( defaultValue = "${project.artifactId}", property="comparisonArtifactId" )
+    protected String comparisonArtifactId;
+
+    /**
+     * Version to compare the current code against.
+     */
+    @Parameter( defaultValue = "(,${project.version})", property="comparisonVersion" )
     protected String comparisonVersion;
 
     /**
-     * Classifier for the artifact to compare the current code against.
-     *
-     * @parameter expression="${comparisonClassifier}"
+     * Artifact to compare the current code against.
      */
+    @Parameter( defaultValue = "${project.packaging}", property="comparisonPackaging" )
+    protected String comparisonPackaging;
+
+    /**
+     * Classifier for the artifact to compare the current code against.
+     */
+    @Parameter( property="comparisonClassifier" )
     protected String comparisonClassifier;
 
     /**
      * A list of packages filter, if empty the whole bundle will be traversed. Values are specified in OSGi package
      * instructions notation, e.g. <code>!org.apache.felix.bundleplugin</code>.
-     *
-     * @parameter
      */
+    @Parameter
     private String[] filters;
 
     /**
      * Project types which this plugin supports.
-     *
-     * @parameter
      */
-    protected List supportedProjectTypes = Arrays.asList( new String[] { "jar", "bundle" } );
+    @Parameter
+    protected List<String> supportedProjectTypes = Arrays.asList( new String[] { "jar", "bundle" } );
 
     public final void execute()
         throws MojoExecutionException, MojoFailureException
+    {
+        this.execute(null);
+    }
+
+    protected void execute( Object context)
+            throws MojoExecutionException, MojoFailureException
     {
         if ( skip )
         {
@@ -183,7 +178,18 @@ abstract class AbstractBaselinePlugin
             return;
         }
 
-        final Jar previousBundle = getPreviousBundle();
+        final Artifact previousArtifact = getPreviousArtifact();
+
+        final Jar previousBundle;
+        if (previousArtifact != null)
+        {
+            previousBundle = openJar(previousArtifact.getFile());
+        }
+        else
+        {
+            previousBundle = null;
+        }
+
         if ( previousBundle == null )
         {
             getLog().info( "Not generating Baseline report as there is no previous version of the library to compare against" );
@@ -202,38 +208,46 @@ abstract class AbstractBaselinePlugin
             packageFilters = new Instructions( Arrays.asList( filters ) );
         }
 
-        // go!
-
-        init();
 
         String generationDate = new SimpleDateFormat( "yyyy-MM-dd'T'HH:mm'Z'" ).format( new Date() );
-        Reporter reporter = new Processor();
+        final Reporter reporter = new Processor();
+
+        final Info[] infos;
+        try
+        {
+            final Set<Info> infoSet = new Baseline( reporter, new DiffPluginImpl() )
+                                .baseline( currentBundle, previousBundle, packageFilters );
+            infos = infoSet.toArray( new Info[infoSet.size()] );
+            Arrays.sort( infos, new InfoComparator() );
+        }
+        catch ( final Exception e )
+        {
+            throw new MojoExecutionException( "Impossible to calculate the baseline", e );
+        }
+        finally
+        {
+            closeJars( currentBundle, previousBundle );
+        }
 
         try
         {
-            Set<Info> infoSet = new Baseline( reporter, new DiffPluginImpl() )
-                                .baseline( currentBundle, previousBundle, packageFilters );
+            // go!
+            context = this.init(context);
+            startBaseline( context, generationDate, project.getArtifactId(), project.getVersion(), previousArtifact.getVersion() );
 
-            startBaseline( generationDate, project.getArtifactId(), project.getVersion(), comparisonVersion );
-
-            final Info[] infos = infoSet.toArray( new Info[infoSet.size()] );
-            Arrays.sort( infos, new InfoComparator() );
-
-            for ( Info info : infos )
+            for ( final Info info : infos )
             {
                 DiffMessage diffMessage = null;
-                Version newerVersion = info.newerVersion;
-                Version suggestedVersion = info.suggestedVersion;
 
-                if ( suggestedVersion != null )
+                if ( info.suggestedVersion != null )
                 {
-                    if ( newerVersion.compareTo( suggestedVersion ) > 0 )
+                    if ( info.newerVersion.compareTo( info.suggestedVersion ) > 0 )
                     {
                         diffMessage = new DiffMessage( "Excessive version increase", DiffMessage.Type.warning );
                         reporter.warning( "%s: %s; detected %s, suggested %s",
                                           info.packageName, diffMessage, info.newerVersion, info.suggestedVersion );
                     }
-                    else if ( newerVersion.compareTo( suggestedVersion ) < 0 )
+                    else if ( info.newerVersion.compareTo( info.suggestedVersion ) < 0 )
                     {
                         diffMessage = new DiffMessage( "Version increase required", DiffMessage.Type.error );
                         reporter.error( "%s: %s; detected %s, suggested %s",
@@ -241,16 +255,10 @@ abstract class AbstractBaselinePlugin
                     }
                 }
 
-                Diff packageDiff = info.packageDiff;
-
-                Delta delta = packageDiff.getDelta();
-
-                switch ( delta )
+                switch ( info.packageDiff.getDelta() )
                 {
                     case UNCHANGED:
-                        if ( ( suggestedVersion.getMajor() != newerVersion.getMajor() )
-                            || ( suggestedVersion.getMicro() != newerVersion.getMicro() )
-                            || ( suggestedVersion.getMinor() != newerVersion.getMinor() ) )
+                        if ( info.newerVersion.compareTo( info.suggestedVersion ) != 0 )
                         {
                             diffMessage = new DiffMessage( "Version has been increased but analysis detected no changes", DiffMessage.Type.warning );
                             reporter.warning( "%s: %s; detected %s, suggested %s",
@@ -273,42 +281,30 @@ abstract class AbstractBaselinePlugin
                         break;
                 }
 
-                boolean mismatch = info.mismatch;
-                String packageName = info.packageName;
-                String shortDelta = getShortDelta( info.packageDiff.getDelta() );
-                String deltaString = StringUtils.lowerCase( String.valueOf( info.packageDiff.getDelta() ) );
-                String newerVersionString = String.valueOf( info.newerVersion );
-                String olderVersionString = String.valueOf( info.olderVersion );
-                String suggestedVersionString = String.valueOf( ( info.suggestedVersion == null ) ? "-" : info.suggestedVersion );
-                Map<String,String> attributes = info.attributes;
-
-                startPackage( mismatch,
-                              packageName,
-                              shortDelta,
-                              deltaString,
-                              newerVersionString,
-                              olderVersionString,
-                              suggestedVersionString,
+                startPackage( context,
+                              info.mismatch,
+                              info.packageName,
+                              getShortDelta( info.packageDiff.getDelta() ),
+                              StringUtils.lowerCase( String.valueOf( info.packageDiff.getDelta() ) ),
+                              info.newerVersion,
+                              info.olderVersion,
+                              info.suggestedVersion,
                               diffMessage,
-                              attributes );
+                              info.attributes );
 
-                if ( Delta.REMOVED != delta )
+                if ( Delta.REMOVED != info.packageDiff.getDelta() )
                 {
-                    doPackageDiff( packageDiff );
+                    doPackageDiff( context, info.packageDiff );
                 }
 
-                endPackage();
+                endPackage(context);
             }
 
-            endBaseline();
-        }
-        catch ( Exception e )
-        {
-            throw new MojoExecutionException( "Impossible to calculate the baseline", e );
+            endBaseline(context);
         }
         finally
         {
-            closeJars( currentBundle, previousBundle );
+            this.close(context);
         }
 
         // check if it has to fail if some error has been detected
@@ -353,7 +349,7 @@ abstract class AbstractBaselinePlugin
         }
     }
 
-    private void doPackageDiff( Diff diff )
+    private void doPackageDiff( Object context, Diff diff )
     {
         int depth = 1;
 
@@ -361,58 +357,62 @@ abstract class AbstractBaselinePlugin
         {
             if ( Delta.UNCHANGED != curDiff.getDelta() )
             {
-                doDiff( curDiff, depth );
+                doDiff( context, curDiff, depth );
             }
         }
     }
 
-    private void doDiff( Diff diff, int depth )
+    private void doDiff( Object context, Diff diff, int depth )
     {
         String type = StringUtils.lowerCase( String.valueOf( diff.getType() ) );
         String shortDelta = getShortDelta( diff.getDelta() );
         String delta = StringUtils.lowerCase( String.valueOf( diff.getDelta() ) );
         String name = diff.getName();
 
-        startDiff( depth, type, name, delta, shortDelta );
+        startDiff( context, depth, type, name, delta, shortDelta );
 
         for ( Diff curDiff : diff.getChildren() )
         {
             if ( Delta.UNCHANGED != curDiff.getDelta() )
             {
-                doDiff( curDiff, depth + 1 );
+                doDiff( context, curDiff, depth + 1 );
             }
         }
 
-        endDiff( depth );
+        endDiff( context, depth );
     }
 
     // extensions APIs
 
-    protected abstract void init();
+    protected abstract Object init(final Object initialContext);
 
-    protected abstract void startBaseline( String generationDate, String bundleName, String currentVersion, String previousVersion );
+    protected abstract void close(final Object context);
 
-    protected abstract void startPackage( boolean mismatch,
+    protected abstract void startBaseline( final Object context, String generationDate, String bundleName, String currentVersion, String previousVersion );
+
+    protected abstract void startPackage( final Object context,
+            boolean mismatch,
                                           String name,
                                           String shortDelta,
                                           String delta,
-                                          String newerVersion,
-                                          String olderVersion,
-                                          String suggestedVersion,
+                                          Version newerVersion,
+                                          Version olderVersion,
+                                          Version suggestedVersion,
                                           DiffMessage diffMessage,
                                           Map<String,String> attributes );
 
-    protected abstract void startDiff( int depth,
+    protected abstract void startDiff( final Object context,
+                                       int depth,
                                        String type,
                                        String name,
                                        String delta,
                                        String shortDelta );
 
-    protected abstract void endDiff( int depth );
+    protected abstract void endDiff( final Object context, int depth );
 
-    protected abstract void endPackage();
+    protected abstract void endPackage(final Object context);
 
-    protected abstract void endBaseline();
+    protected abstract void endBaseline(final Object context);
 
     // internals
 
@@ -434,7 +434,7 @@ abstract class AbstractBaselinePlugin
         return openJar( currentBundle );
     }
 
-    private Jar getPreviousBundle()
+    private Artifact getPreviousArtifact()
         throws MojoFailureException, MojoExecutionException
     {
         // Find the previous version JAR and resolve it, and it's dependencies
@@ -452,10 +452,10 @@ abstract class AbstractBaselinePlugin
         try
         {
             previousArtifact =
-                factory.createDependencyArtifact( project.getGroupId(),
-                                                  project.getArtifactId(),
+                factory.createDependencyArtifact( comparisonGroupId,
+                                                  comparisonArtifactId,
                                                   range,
-                                                  project.getPackaging(),
+                                                  comparisonPackaging,
                                                   comparisonClassifier,
                                                   Artifact.SCOPE_COMPILE );
 
@@ -465,7 +465,7 @@ abstract class AbstractBaselinePlugin
                 @SuppressWarnings( "unchecked" )
                 // type is konwn
                 List<ArtifactVersion> availableVersions =
-                    metadataSource.retrieveAvailableVersions( previousArtifact, localRepository,
+                    metadataSource.retrieveAvailableVersions( previousArtifact, session.getLocalRepository(),
                                                               project.getRemoteArtifactRepositories() );
                 filterSnapshots( availableVersions );
                 ArtifactVersion version = range.matchVersion( availableVersions );
@@ -492,11 +492,11 @@ abstract class AbstractBaselinePlugin
 
         try
         {
-            resolver.resolve( previousArtifact, project.getRemoteArtifactRepositories(), localRepository );
+            resolver.resolve( previousArtifact, project.getRemoteArtifactRepositories(), session.getLocalRepository() );
         }
         catch ( ArtifactResolutionException are )
         {
-            throw new MojoExecutionException( "Artifact " + previousArtifact + " cannot be resolved", are );
+            throw new MojoExecutionException( "Artifact " + previousArtifact + " cannot be resolved : " + are.getMessage(), are );
         }
         catch ( ArtifactNotFoundException anfe )
         {
@@ -504,7 +504,7 @@ abstract class AbstractBaselinePlugin
                 + " does not exist on local/remote repositories", anfe );
         }
 
-        return openJar( previousArtifact.getFile() );
+        return previousArtifact;
     }
 
     private void filterSnapshots( List<ArtifactVersion> versions )
@@ -512,27 +512,27 @@ abstract class AbstractBaselinePlugin
         for ( Iterator<ArtifactVersion> versionIterator = versions.iterator(); versionIterator.hasNext(); )
         {
             ArtifactVersion version = versionIterator.next();
-            if ( "SNAPSHOT".equals( version.getQualifier() ) )
+            if ( version.getQualifier() != null && version.getQualifier().endsWith( "SNAPSHOT" ) )
             {
                 versionIterator.remove();
             }
         }
     }
 
-    private static Jar openJar( File file )
+    private static Jar openJar( final File file )
         throws MojoExecutionException
     {
         try
         {
             return new Jar( file );
         }
-        catch ( IOException e )
+        catch ( final IOException e )
         {
             throw new MojoExecutionException( "An error occurred while opening JAR directory: " + file, e );
         }
     }
 
-    private static void closeJars( Jar...jars )
+    private static void closeJars( final Jar...jars )
     {
         for ( Jar jar : jars )
         {
@@ -596,5 +596,4 @@ abstract class AbstractBaselinePlugin
                 return String.valueOf( deltaString.charAt( 0 ) );
         }
     }
-
 }
